---------------------------------------------------------------
--                                                           
--  PENCIL AND PAPER INTERPRETER
--  By: Martin Carlisle and Al White     
--                                                           
--  Controls
--  Description : Handle actions on controls in main window 
--                       
--  By: Martin Carlisle
--                                         
-- PENCIL AND PAPER INTERPRETER is free software; you can redistribute 
-- it and/or modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- PENCIL AND PAPER INTERPRETER is distributed in the hope that it will 
-- be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------

with Simulation_Gui;
with State;
with Mcc.Gui.Widget.Button.Picture;
with Main_Window;
with Mcc.Gui.Event_Scheduler;
with Mcc.Gui.Widget.Scale;
with Pencil_And_Paper_Computer;
with Read_Dialog_Window;
with Read_Dialog;
with Mcc.Gui.Widget.Listbox;
package body Controls is
   procedure Run_Timer_Callback;
   
   Speed_Constant_Part : constant := 50;
   Speed_Slope_Part    : constant := 30;
   Running : Boolean := False;
   Reading : Boolean := False;
    
   procedure Read_Callback(
      Post_Process : in Pencil_And_Paper_Computer.Post_Read_Callback_Type) is
   begin
      Read_Dialog.Post_Process := Post_Process;
      Read_Dialog_Window.Generate_Window;
      Read_Dialog.Initialize;
      Reading := True;
   end Read_Callback;
   
   procedure Read_Complete is
      Scale_Value : Integer;
   begin
      if not State.End_Of_Program then
         Scale_Value := Mcc.Gui.Widget.Scale.Get(
            Main_Window.Speed_Scale);
         Mcc.Gui.Event_Scheduler.Schedule_Event(
            Number_Of_Milliseconds => 
               Speed_Constant_Part+(100-Scale_Value)*Speed_Slope_Part,
            Event_Callback         => Run_Timer_Callback'Access);
      end if;
      Reading := False;
      Simulation_Gui.Draw_PC;  
   end Read_Complete;
   
   procedure Print_Callback(X : in Pencil_And_Paper_Computer.Memory_Value) is
   begin
      Mcc.Gui.Widget.Listbox.Add_Entry(
         Obj      => Main_Window.Output,
         Location => Mcc.Gui.Widget.Listbox.Count(Main_Window.Output),
         Text     => Integer'Image(X));
   end Print_Callback;
   
   procedure Do_Next is
   begin
      State.End_Of_Program := False;
      begin
         Pencil_And_Paper_Computer.Do_One_Instruction(
            Update_Callback => Simulation_Gui.Set_Instruction'Access,
            Read_Callback   => Read_Callback'access,
            Print_Callback  => Print_Callback'access);
      exception
         when Pencil_And_Paper_Computer.Memory_Overflow => 
            Mcc.Gui.Widget.Listbox.Add_Entry(
               Obj      => Main_Window.Output,
               Location => Mcc.Gui.Widget.Listbox.Count(Main_Window.Output),
               Text     => "WARNING! Exceeded largest value");
         when Pencil_And_Paper_Computer.Memory_Underflow => 
            Mcc.Gui.Widget.Listbox.Add_Entry(
               Obj      => Main_Window.Output,
               Location => Mcc.Gui.Widget.Listbox.Count(Main_Window.Output),
               Text     => "WARNING! Negative value set to 0");
      end;
      Simulation_Gui.Draw_PC;  
   exception when others =>
      State.End_Of_Program := True;
   end Do_Next;

   procedure Run_Timer_Callback is
      Scale_Value : Integer;
   begin
      if Running then
         Do_Next;
         if not Reading and not State.End_Of_Program then
            Scale_Value := Mcc.Gui.Widget.Scale.Get(
               Main_Window.Speed_Scale);
            Mcc.Gui.Event_Scheduler.Schedule_Event(
               Number_Of_Milliseconds => 
                  Speed_Constant_Part+(100-Scale_Value)*Speed_Slope_Part,
               Event_Callback         => Run_Timer_Callback'Access);
         end if;
      end if;
   end Run_Timer_Callback;

   -------------
   -- Do_Next --
   -------------

   procedure Do_Next (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Do_Next;
   end Do_Next;

   -----------------
   -- Reset_Robot --
   -----------------

   procedure Reset (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Pencil_And_Paper_Computer.Reset_Program_Counter;
      Mcc.Gui.Widget.Listbox.Clear(Main_Window.Output);
      Simulation_Gui.Draw_PC;
   end Reset;


   --------------------
   -- Run_Until_Stop --
   --------------------

   procedure Run_Until_Stop
     (Obj : in out Mcc.Gui.Widget.Button.Button'Class)
   is
   begin
      Running := True;
      Run_Timer_Callback;
   end Run_Until_Stop;

   ----------
   -- Stop --
   ----------

   procedure Stop is
   begin
      Running := False;
   end Stop;


   procedure Stop (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Stop;
   end Stop;



end Controls;

