---------------------------------------------------------------
--                                                           
--  PENCIL AND PAPER INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  FILE_MENU 
--  Description : Handle choices from File menu 
--                       
--  By: Martin Carlisle
--                                         
-- PENCIL AND PAPER INTERPRETER is free software; you can redistribute 
-- it and/or modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- PENCIL AND PAPER INTERPRETER is distributed in the hope that it will 
-- be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
--
---------------------------------------------------------------
with Mcc.Common_Dialogs;
with Ada.Exceptions;
with Limits;
with Pencil_And_Paper_Computer;
with State;
with Simulation_Gui;
with Mcc.Gui.Event_Scheduler;
with Mcc.Gui.Container.Window;
with Main_Window;
package body File_Menu is
   -- used in File Open/Save dialogs to select
   -- which files are displayed
   File_Types : constant String :=
      "{ ""Pencil and Paper Files"" { .pnp } } " &
      "{ ""All Files"" * }";

   -- Holds file name and Directory name returned from
   -- File dialogs
   Filename       : String(1..Limits.Max_Name_Length);
   Filename_Last  : Natural := 0;
   Directory      : String(1..Limits.Max_Name_Length);
   Directory_Last : Natural;


   -----------------
   -- Exit_Choice --
   -----------------

   procedure Exit_Choice is
   begin
      if State.Program_Modified then
         Save_Program_Choice;
      end if;
      Mcc.Common_Dialogs.Quit_Dialog(Verify => False);
   end Exit_Choice;

   -----------------
   -- Exit_Choice --
   -----------------

   procedure Exit_Choice
     (Obj : in out Mcc.Gui.Container.Window.Window'Class)
   is
   begin
      Exit_Choice;
   end Exit_Choice;

   -------------------------
   -- Load_Program_Choice --
   -------------------------
   procedure Open_File(Name : in String) is
   begin
      Filename_Last := Name'Length;
      Filename(1..Filename_Last) := Name;
      if Filename_Last > 0 then
         -- load this window
         Pencil_And_Paper_Computer.Read_Program(
            Filename => Filename(Filename'First..Filename_Last));
         Pencil_And_Paper_Computer.Reset_Program_Counter;
         Simulation_Gui.Display_Instructions;
         State.Program_Modified := False;
         Mcc.Gui.Container.Window.Set_Title(
            Obj   => Main_Window.Main_Window,
            Title => "Pencil and Paper Simulator: " & 
               Filename(Filename'First..Filename_Last));
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Open_File;
   
   procedure Load_Program_Choice is
   begin
      State.Set_Dialog_Displayed;
      Mcc.Common_Dialogs.Open_Dialog(
         File_Types => File_Types,
         Filename   => Filename,
         File_Last  => Filename_Last,
         Directory  => Directory,
         Dir_Last   => Directory_Last);
      Mcc.Gui.Event_Scheduler.Schedule_Event(500,State.Clear_Dialog_Flag'access);
      if Filename_Last > 0 then
         -- load this window
         Pencil_And_Paper_Computer.Read_Program(
            Filename => Filename(Filename'First..Filename_Last));
         Pencil_And_Paper_Computer.Reset_Program_Counter;
         Simulation_Gui.Display_Instructions;
         State.Program_Modified := False;
         Mcc.Gui.Container.Window.Set_Title(
            Obj   => Main_Window.Main_Window,
            Title => "Pencil and Paper Simulator: " & 
               Filename(Filename'First..Filename_Last));
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Load_Program_Choice;

   procedure Save_Choice is
   begin
      if Filename_Last > 0 then
         Pencil_And_Paper_Computer.Write_Program(
            Filename => Filename(Filename'First..Filename_Last));
         State.Program_Modified := False;
      else
         Save_Program_Choice;
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Save_Choice;
   
   -------------------------
   -- Save_Program_Choice --
   -------------------------

   procedure Save_Program_Choice is
   begin
      State.Set_Dialog_Displayed;
      Mcc.Common_Dialogs.Save_Dialog(
         File_Types => File_Types,
         Filename   => Filename,
         File_Last  => Filename_Last,
         Directory  => Directory,
         Dir_Last   => Directory_Last);
      Mcc.Gui.Event_Scheduler.Schedule_Event(500,State.Clear_Dialog_Flag'access);
      if Filename_Last > 0 then
         Pencil_And_Paper_Computer.Write_Program(
            Filename => Filename(Filename'First..Filename_Last));
         State.Program_Modified := False;
         Mcc.Gui.Container.Window.Set_Title(
            Obj   => Main_Window.Main_Window,
            Title => "Pencil and Paper Simulator: " & 
               Filename(Filename'First..Filename_Last));
      end if;
   exception
      when e:others =>
         Mcc.Common_Dialogs.OK_Box("Unable to open: " &
            Filename(1..Filename_Last) & ASCII.LF &
            "Not a valid Robot file?" & ASCII.LF &
            Ada.Exceptions.Exception_Information(E));
   end Save_Program_Choice;

   ---------------------------------------
   -- add an instruction at the current PC
   ---------------------------------------
   procedure Insert_Instruction is
   begin
      Pencil_And_Paper_Computer.Insert_Instruction;
      Simulation_Gui.Display_Instructions;
   end Insert_Instruction;
   -------------------------------------------
   -- delete the instruction at the current PC
   -------------------------------------------
   procedure Delete_Instruction is
   begin
      Pencil_And_Paper_Computer.Delete_Instruction;
      Simulation_Gui.Display_Instructions;
   end Delete_Instruction;
end File_Menu;

