---------------------------------------------------------------
--                                                           
--  ROBOT INTERPRETER
--  By: Martin Carlisle and Keith Shomper     
--                                                           
--  PC_Dialog 
--  Description : procedures for dialog to change Program Counter
--                       
--  By: Martin Carlisle
--                                         
-- ROBOT INTERPRETER is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--  
-- ROBOT INTERPRETER is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------

with Pencil_And_Paper_Computer;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with State;
with Simulation_Gui;
with Get_PC_Window;
with Instruction_Dialog;
with Mcc.Gui.Widget.Text_Entry;
with Mcc.Gui.Container.Window;
with Mcc.Gui.Keys;
with Controls;
with Mcc.Gui.Widget.Label;
package body PC_Dialog is
   type Which_Control is (PC,Accumulator);
   Remember_Which : Which_Control;
   --------
   -- Ok --
   --------
   procedure Ok is
      Good_String : Boolean := True;
      Text_Size : Natural;
   begin
      case Remember_Which is
         when PC => Text_Size := 5;
         when Accumulator => Text_Size := 14;
      end case;
      Get_PC_Window.Read_Window(Good_String);
      if Length(PC_String)=Text_Size then
         declare
            Last : constant Integer := Text_Size+3;
            Value : String(1..Last) := (
               1 => '2',
               2 => '#',
               Others => '0');
         begin
            Value(Last) := '#';
            Value(3..Text_Size+2) := To_String(PC_String);
            for I in 3..Text_Size+2 loop
               if Value(I)/='0' and Value(I)/='1' then
                  Good_String := False;
                  exit;
               end if;
            end loop;
         end;
      else
         Good_String := False;
      end if;
      if not Good_String then
         Mcc.Gui.Bell;
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Get_PC_Window.PC_Text);
      else
         case Remember_Which is
            when PC =>
               -- if we were ended, reset to running before
               -- setting actual PC
               Pencil_And_Paper_Computer.Reset_Program_Counter;
               Pencil_And_Paper_Computer.Set_Program_Counter(
                  Address => Instruction_Dialog.To_Number(
                     To_String(PC_String)));
            when Accumulator =>
               Pencil_And_Paper_Computer.Set_Accumulator(
                  Value => Instruction_Dialog.To_Number(
                     To_String(PC_String)));
         end case;
         Simulation_Gui.Draw_PC;
         Mcc.Gui.Container.Window.Destroy(
            Get_PC_Window.Get_PC_Window);
      end if;
   end Ok;
   
   procedure Ok (Obj : in out Mcc.Gui.Widget.Button.Button'Class) is
   begin
      Ok;
   end Ok;


   procedure Key_Callback (
      Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Key_Event) is
      use Mcc.Gui;
      Text_Size : Natural;
   begin
      case Remember_Which is
         when PC => Text_Size := 5;
         when Accumulator => Text_Size := 14;
      end case;
      if Event.Action=Release and Event.Key=Mcc.Gui.Keys.Enter then
         Ok;
      elsif Event.Action=Release and Mcc.Gui.Widget.Text_Entry.Get_Text_Length(
         Get_PC_Window.PC_Text) > Text_Size then
         Mcc.Gui.Bell;
         declare
            Current : String := Mcc.Gui.Widget.Text_Entry.Get_Text(
               Get_PC_Window.PC_Text);
         begin
            Mcc.Gui.Widget.Text_Entry.Set_Text(
               Obj => Get_PC_Window.PC_Text,
               Text => Current(Current'First..Current'First+Text_Size-1));
         end;
      end if;
   end Key_Callback;


   --------------
   -- PC_Click --
   --------------

   procedure Click
     (Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event;
      Which : in     Which_Control)
   is
      use Mcc.Gui;
      PC_Value : Natural;
      Accumulator_Value : Natural;
      Window_X : Integer := 250;
      Window_Y : Integer := 250;
   begin
      if State.Dialog_Displayed then
         return;
      end if;
      Remember_Which := Which;
      if Event.Button=Left and
         Event.Action=Release  then
         Controls.Stop;
         case Which is
            when PC =>
               PC_Value := Pencil_And_Paper_Computer.Get_Program_Counter;
               PC_String := To_Unbounded_String(
                  Simulation_Gui.Five_Bit_Binary(PC_Value));
            when Accumulator =>
               Accumulator_Value := Pencil_And_Paper_Computer.Get_Accumulator;
               PC_String := To_Unbounded_String(
                  Simulation_Gui.Five_Bit_Binary(Accumulator_Value));
         end case;
         Get_PC_Window.Generate_And_Fill_Window;
         case Which is
            when PC =>
               null;
            when Accumulator =>
               Mcc.Gui.Widget.Label.Set_Text(
                  Obj => Get_PC_Window.Label1,
                  Text => "Enter 14 bit accumulator value");
         end case;
         Mcc.Gui.Container.Window.Move(
            Obj => Get_PC_Window.Get_PC_Window,
            X   => Window_X,
            Y   => Window_Y);
         Mcc.Gui.Widget.Text_Entry.Highlight(
            Get_PC_Window.PC_Text);
         Mcc.Gui.Set_Key_Listener(
            Obj      => Get_PC_Window.PC_Text'access,
            Listener => Key_Callback'access);
      end if;
   end Click;
   
   procedure PC_Click
     (Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
   begin
      Click(Obj,Event,PC);
   end PC_Click;

   procedure Accumulator_Click
     (Obj   : in out Mcc.Gui.Sized_Object'Class;
      Event : in     Mcc.Gui.Mouse_Event) is
   begin
      Click(Obj,Event,Accumulator);
   end Accumulator_Click;

end PC_Dialog;

